=begin
Copyright 2015, Rick Reed
All Rights Reserved THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
License: N/A
Author: Rick Reed
Organization: 
Name: DrawWhorl2
Version: 1.0.6
SU Version: 2015
Date: 11/29/2015 
Description: Draws screw and bolt threads using fixed and tapered sections.
Usage: Enter start and end radii for tapered section, and number of turns for fixed radius and tapered radius section. Also enter thread @@WRLpitch, thread width, height, and edge height.
History:
1.0.0 2015-11-19 First version Not Published
1.0.3 2015-11-25 Restructured package as per Extension Warehouse review Not Published
1.0.6 2015-11-29 Changed tooth to thread, and thread parameters to float
=end

require "sketchup.rb"
# Main code (start module name with capital letter)
module DrawWhorl2
	
	@@WRLeradius = nil
	@@WRLsradius = nil
	@@WRLpitch = nil
	@@WRLrotations = nil
	@@WRLsrrotations = nil
	@@WRLsections = nil
	@@WRLthreadheight = nil
	@@WRLthreadlength = nil
	@@WRLedgeheight = nil
		
	def self.drawWhorl_w_dialog

		# chosen_folder = UI.select_directory(title: "Select Image Directory")
	 
		prompts = ["End Radius","Start Radius", "Pitch", "No of Tapered R Rots", "No of Fixed R Rots", "Sections per Rotation ", "Thread Height", "Thread Length", "Thread Edge Height"]
		if @@WRLeradius.nil?
			values = [1.0.inch, 0.75.inch, 0.25.inch, 5, 5, 24, 0.125.inch, 0.25.inch, 0.125.inch]
		else
			values = [@@WRLeradius, @@WRLsradius, @@WRLpitch, @@WRLrotations, @@WRLsrrotations, @@WRLsections, @@WRLthreadheight, @@WRLthreadlength, @@WRLedgeheight]
		end
		results = inputbox prompts, values, "Whorl Dimensions"
		return if not results
		@@WRLeradius, @@WRLsradius, @@WRLpitch, @@WRLrotations, @@WRLsrrotations, @@WRLsections, @@WRLthreadheight, @@WRLthreadlength, @@WRLedgeheight = results

		totalsec = @@WRLsections * @@WRLrotations
		totalsrsec = @@WRLsections * @@WRLsrrotations
	
		angle = 2*3.14159/@@WRLsections
		cosangle = Math.cos(angle)
		sinangle = Math.sin(angle)

		section = 1
		z0 = @@WRLpitch / @@WRLsections
	
		width = 0;
		depth = @@WRLthreadlength;
		height = @@WRLthreadheight / 2.0;
		edgeoffset = @@WRLedgeheight / 2.0;

		r1 = @@WRLsradius
		r2 = @@WRLsradius - depth;
		if totalsec > 0
			dr = (@@WRLeradius - @@WRLsradius) / totalsec
		else
			dr = 0
		end

		pts = []
		base = []
		x1 = r1
		y1 = 0
		z1 = 0
	
		pts[pts.length] = [x1,y1,z1]
	
		model = Sketchup.active_model
		entities = model.active_entities
		group = entities.add_group
		entities = group.entities
		model.start_operation "DrawWhorl1"	

		if edgeoffset > 0

			while section < (totalsec + 1)
				section2 = section - 1
	
				p1x = (r1 + (dr * section2)) * Math.cos(section2 * angle)
				p1y = (r1 + (dr * section2)) * Math.sin(section2 * angle)
				p1z = (section2 * z0) + edgeoffset
				p1z2 = (section2 * z0) - edgeoffset
		
				p2x = (r2 + (dr * section2)) * Math.cos(section2 * angle)
				p2y = (r2 + (dr * section2)) * Math.sin(section2 * angle)
				p2z = (section2 * z0) + height
				p2z2 = (section2 * z0) - height

				p3x = (r2 + (dr * (section2 + 1))) * Math.cos((section2 + 1) * angle)
				p3y = (r2 + (dr * (section2 + 1))) * Math.sin((section2 + 1) * angle)
				p3z = ((section2 + 1) * z0) + height
				p3z2 = ((section2 + 1) * z0) - height

				p4x = (r1 + (dr * (section2 + 1))) * Math.cos((section2 + 1) * angle)
				p4y = (r1 + (dr * (section2 + 1))) * Math.sin((section2 + 1) * angle)
				p4z = ((section2 + 1) * z0) + edgeoffset;
				p4z2 = ((section2 + 1) * z0) - edgeoffset;

				pts1 = []
				pts1[0] = [p1x, p1y, p1z] # p1t
				pts1[1] = [p2x, p2y, p2z] # p2t
				pts1[2] = [p3x, p3y, p3z] # p3t
				base[section] = entities.add_face pts1
		
				pts2 = []
				pts2[0] = [p3x, p3y, p3z] # p3t
				pts2[1] = [p4x, p4y, p4z] # p4t
				pts2[2] = [p1x, p1y, p1z] # p1t
				base[section] = entities.add_face pts2
	
				pts3 = []
				pts3[0] = [p1x, p1y, p1z2] # p1b
				pts3[1] = [p2x, p2y, p2z2] # p2b
				pts3[2] = [p3x, p3y, p3z2] # p3b
				base[section] = entities.add_face pts3
		
				pts4 = []
				pts4[0] = [p3x, p3y, p3z2] # p3b
				pts4[1] = [p4x, p4y, p4z2] # p4b
				pts4[2] = [p1x, p1y, p1z2] # p1b
				base[section] = entities.add_face pts4
	
				# p1b p1t p4b
				# p1t p4t p4b
		
				pts5 = []
				pts4[0] = [p1x, p1y, p1z2] # p1b
				pts4[1] = [p1x, p1y, p1z] # p1t
				pts4[2] = [p4x, p4y, p4z2] # p4b
				base[section] = entities.add_face pts4
		
				pts6 = []
				pts6[0] = [p1x, p1y, p1z] # p1t
				pts6[1] = [p4x, p4y, p4z] # p4t
				pts6[2] = [p4x, p4y, p4z2] # p4b
				base[section] = entities.add_face pts6
	
				section += 1
			end

			while section < (totalsrsec + totalsec + 1) 
				section2 = section - 1
			
				p1x = (r1 + (dr * totalsec)) * Math.cos(section2 * angle)
				p1y = (r1 + (dr * totalsec)) * Math.sin(section2 * angle)
				p1z = section2 * z0 + edgeoffset
				p1z2 = section2 * z0 - edgeoffset
		
				p2x = (r2 + (dr * totalsec)) * Math.cos(section2 * angle)
				p2y = (r2 + (dr * totalsec)) * Math.sin(section2 * angle)
				p2z = (section2 * z0) + height
				p2z2 = (section2 * z0) - height

				p3x = (r2 + (dr * totalsec)) * Math.cos((section2 + 1) * angle)
				p3y = (r2 + (dr * totalsec)) * Math.sin((section2 + 1) * angle)
				p3z = ((section2 + 1) * z0) + height
				p3z2 = ((section2 + 1) * z0) - height

				p4x = (r1 + (dr * totalsec)) * Math.cos((section2 + 1) * angle)
				p4y = (r1 + (dr * totalsec)) * Math.sin((section2 + 1) * angle)
				p4z = (section2 + 1) * z0 + edgeoffset
				p4z2 = (section2 + 1) * z0 - edgeoffset

				pts1 = []
				pts1[0] = [p1x, p1y, p1z] # p1t
				pts1[1] = [p2x, p2y, p2z] # p2t
				pts1[2] = [p3x, p3y, p3z] # p3t
				base[section] = entities.add_face pts1
		
				pts2 = []
				pts2[0] = [p3x, p3y, p3z] # p3t
				pts2[1] = [p4x, p4y, p4z] # p4t
				pts2[2] = [p1x, p1y, p1z] # p1t
				base[section] = entities.add_face pts2
	
				pts3 = []
				pts3[0] = [p1x, p1y, p1z2] # p1b
				pts3[1] = [p2x, p2y, p2z2] # p2b
				pts3[2] = [p3x, p3y, p3z2] # p3b
				base[section] = entities.add_face pts3
		
				pts4 = []
				pts4[0] = [p3x, p3y, p3z2] # p3b
				pts4[1] = [p4x, p4y, p4z2] # p4b
				pts4[2] = [p1x, p1y, p1z2] # p1b
				base[section] = entities.add_face pts4
	
				# p1b p1t p4b
				# p1t p4t p4b
		
				pts5 = []
				pts4[0] = [p1x, p1y, p1z2] # p1b
				pts4[1] = [p1x, p1y, p1z] # p1t
				pts4[2] = [p4x, p4y, p4z2] # p4b
				base[section] = entities.add_face pts4
		
				pts6 = []
				pts6[0] = [p1x, p1y, p1z] # p1t
				pts6[1] = [p4x, p4y, p4z] # p4t
				pts6[2] = [p4x, p4y, p4z2] # p4b
				base[section] = entities.add_face pts6
	
				section += 1
			end

		else

			while section < (totalsec + 1) 
				section2 = section - 1

				p1x = (r1 + (dr * section2)) * Math.cos(section2 * angle)
				p1y = (r1 + (dr * section2)) * Math.sin(section2 * angle)
				p1z = section2 * z0
		
				p2x = (r2 + (dr * section2)) * Math.cos(section2 * angle)
				p2y = (r2 + (dr * section2)) * Math.sin(section2 * angle)
				p2z = (section2 * z0) + height
				p2z2 = (section2 * z0) - height

				p3x = (r2 + (dr * (section2 + 1))) * Math.cos((section2 + 1) * angle)
				p3y = (r2 + (dr * (section2 + 1))) * Math.sin((section2 + 1) * angle)
				p3z = ((section2 + 1) * z0) + height
				p3z2 = ((section2 + 1) * z0) - height

				p4x = (r1 + (dr * (section2 + 1))) * Math.cos((section2 + 1) * angle)
				p4y = (r1 + (dr * (section2 + 1))) * Math.sin((section2 + 1) * angle)
				p4z = (section2 + 1) * z0

				pts1 = []
				pts1[0] = [p1x, p1y, p1z]
				pts1[1] = [p2x, p2y, p2z]
				pts1[2] = [p3x, p3y, p3z]
				base[section] = entities.add_face pts1
		
				pts2 = []
				pts2[0] = [p3x, p3y, p3z]
				pts2[1] = [p4x, p4y, p4z]
				pts2[2] = [p1x, p1y, p1z]
				base[section] = entities.add_face pts2
	
				pts3 = []
				pts3[0] = [p1x, p1y, p1z]
				pts3[1] = [p2x, p2y, p2z2]
				pts3[2] = [p3x, p3y, p3z2]
				base[section] = entities.add_face pts3
		
				pts4 = []
				pts4[0] = [p3x, p3y, p3z2]
				pts4[1] = [p4x, p4y, p4z]
				pts4[2] = [p1x, p1y, p1z]
				base[section] = entities.add_face pts4
	
				section += 1
			end

			while section < (totalsrsec + totalsec + 1) 
				section2 = section - 1
			
				p1x = (r1 + (dr * totalsec)) * Math.cos(section2 * angle)
				p1y = (r1 + (dr * totalsec)) * Math.sin(section2 * angle)
				p1z = section2 * z0
		
				p2x = (r2 + (dr * totalsec)) * Math.cos(section2 * angle)
				p2y = (r2 + (dr * totalsec)) * Math.sin(section2 * angle)
				p2z = (section2 * z0) + height
				p2z2 = (section2 * z0) - height

				p3x = (r2 + (dr * totalsec)) * Math.cos((section2 + 1) * angle)
				p3y = (r2 + (dr * totalsec)) * Math.sin((section2 + 1) * angle)
				p3z = ((section2 + 1) * z0) + height
				p3z2 = ((section2 + 1) * z0) - height

				p4x = (r1 + (dr * totalsec)) * Math.cos((section2 + 1) * angle)
				p4y = (r1 + (dr * totalsec)) * Math.sin((section2 + 1) * angle)
				p4z = (section2 + 1) * z0

				pts1 = []
				pts1[0] = [p1x, p1y, p1z]
				pts1[1] = [p2x, p2y, p2z]
				pts1[2] = [p3x, p3y, p3z]
				base[section] = entities.add_face pts1
		
				pts2 = []
				pts2[0] = [p3x, p3y, p3z]
				pts2[1] = [p4x, p4y, p4z]
				pts2[2] = [p1x, p1y, p1z]
				base[section] = entities.add_face pts2
	
				pts3 = []
				pts3[0] = [p1x, p1y, p1z]
				pts3[1] = [p2x, p2y, p2z2]
				pts3[2] = [p3x, p3y, p3z2]
				base[section] = entities.add_face pts3
		
				pts4 = []
				pts4[0] = [p3x, p3y, p3z2]
				pts4[1] = [p4x, p4y, p4z]
				pts4[2] = [p1x, p1y, p1z]
				base[section] = entities.add_face pts4
	
				section += 1
			end

		end

		model.commit_operation

	end 
	def self.drawWhorl_w_open

		# chosen_folder = UI.select_directory(title: "Select Image Directory")
	
		values = []

		chosen_file = UI.openpanel("Open Whorl Settings File", "", "Text Files|*.txt;||")
		if chosen_file != nil
			lines=IO.readlines(chosen_file)
			i = 1
			lines.each{|line|
				line.chomp! ### removes '\n'
				### process the line as needed
				#UI.messagebox(line)
				if i == 1 
					values.push(line.to_f)
				elsif i == 2 
					values.push(line.to_f)
				elsif i == 3 
					values.push(line.to_f)
				elsif i == 4 
					values.push(line.to_i)
				elsif i == 5 
					values.push(line.to_i)
				elsif i == 6 
					values.push(line.to_i)
				elsif i == 7 
					values.push(line.to_f)
				elsif i == 8 
					values.push(line.to_f)
				elsif i == 9 
					values.push(line.to_f)
				end
				i =+ 1
			}
			# file.close
		else
			values = [10, 10, 4, 5, 5, 24, 2, 5, 1]
		end
 
		prompts = ["End Radius","Start Radius", "Pitch", "No of Tapered R Rots", "No of Fixed R Rots", "Sections per Rotation ", "Thread Height", "Thread Length", "Edge Height"]
		#values = [300.mm, 300.mm, 100.mm, 5, 5, 24, 2, 5, 1]
		results = inputbox prompts, values, "Whorl Dimensions"
		return if not results
		@@WRLeradius, @@WRLsradius, @@WRLpitch, @@WRLrotations, @@WRLsrrotations, @@WRLsections, @@WRLthreadheight, @@WRLthreadlength, @@WRLedgeheight = results

		totalsec = @@WRLsections * @@WRLrotations
		totalsrsec = @@WRLsections * @@WRLsrrotations
	
		angle = 2*3.14159/@@WRLsections
		cosangle = Math.cos(angle)
		sinangle = Math.sin(angle)

		section = 1
		z0 = @@WRLpitch / @@WRLsections
	
		width = 0;
		depth = @@WRLthreadlength;
		height = @@WRLthreadheight / 2.0;
		edgeoffset = @@WRLedgeheight / 2.0;

		r1 = @@WRLsradius
		r2 = @@WRLsradius - depth;
		if totalsec > 0
			dr = (@@WRLeradius - @@WRLsradius) / totalsec
		else
			dr = 0
		end

		pts = []
		base = []
		x1 = r1
		y1 = 0
		z1 = 0
	
		pts[pts.length] = [x1,y1,z1]
	
		model = Sketchup.active_model
		entities = model.active_entities
		group = entities.add_group
		entities = group.entities
		model.start_operation "DrawWhorl1"	

		if edgeoffset > 0

			while section < (totalsec + 1)
				section2 = section - 1
	
				p1x = (r1 + (dr * section2)) * Math.cos(section2 * angle)
				p1y = (r1 + (dr * section2)) * Math.sin(section2 * angle)
				p1z = (section2 * z0) + edgeoffset
				p1z2 = (section2 * z0) - edgeoffset
		
				p2x = (r2 + (dr * section2)) * Math.cos(section2 * angle)
				p2y = (r2 + (dr * section2)) * Math.sin(section2 * angle)
				p2z = (section2 * z0) + height
				p2z2 = (section2 * z0) - height

				p3x = (r2 + (dr * (section2 + 1))) * Math.cos((section2 + 1) * angle)
				p3y = (r2 + (dr * (section2 + 1))) * Math.sin((section2 + 1) * angle)
				p3z = ((section2 + 1) * z0) + height
				p3z2 = ((section2 + 1) * z0) - height

				p4x = (r1 + (dr * (section2 + 1))) * Math.cos((section2 + 1) * angle)
				p4y = (r1 + (dr * (section2 + 1))) * Math.sin((section2 + 1) * angle)
				p4z = ((section2 + 1) * z0) + edgeoffset;
				p4z2 = ((section2 + 1) * z0) - edgeoffset;

				pts1 = []
				pts1[0] = [p1x, p1y, p1z] # p1t
				pts1[1] = [p2x, p2y, p2z] # p2t
				pts1[2] = [p3x, p3y, p3z] # p3t
				base[section] = entities.add_face pts1
		
				pts2 = []
				pts2[0] = [p3x, p3y, p3z] # p3t
				pts2[1] = [p4x, p4y, p4z] # p4t
				pts2[2] = [p1x, p1y, p1z] # p1t
				base[section] = entities.add_face pts2
	
				pts3 = []
				pts3[0] = [p1x, p1y, p1z2] # p1b
				pts3[1] = [p2x, p2y, p2z2] # p2b
				pts3[2] = [p3x, p3y, p3z2] # p3b
				base[section] = entities.add_face pts3
		
				pts4 = []
				pts4[0] = [p3x, p3y, p3z2] # p3b
				pts4[1] = [p4x, p4y, p4z2] # p4b
				pts4[2] = [p1x, p1y, p1z2] # p1b
				base[section] = entities.add_face pts4
	
				# p1b p1t p4b
				# p1t p4t p4b
		
				pts5 = []
				pts4[0] = [p1x, p1y, p1z2] # p1b
				pts4[1] = [p1x, p1y, p1z] # p1t
				pts4[2] = [p4x, p4y, p4z2] # p4b
				base[section] = entities.add_face pts4
		
				pts6 = []
				pts6[0] = [p1x, p1y, p1z] # p1t
				pts6[1] = [p4x, p4y, p4z] # p4t
				pts6[2] = [p4x, p4y, p4z2] # p4b
				base[section] = entities.add_face pts6
	
				section += 1
			end

			while section < (totalsrsec + totalsec + 1) 
				section2 = section - 1
			
				p1x = (r1 + (dr * totalsec)) * Math.cos(section2 * angle)
				p1y = (r1 + (dr * totalsec)) * Math.sin(section2 * angle)
				p1z = section2 * z0 + edgeoffset
				p1z2 = section2 * z0 - edgeoffset
		
				p2x = (r2 + (dr * totalsec)) * Math.cos(section2 * angle)
				p2y = (r2 + (dr * totalsec)) * Math.sin(section2 * angle)
				p2z = (section2 * z0) + height
				p2z2 = (section2 * z0) - height

				p3x = (r2 + (dr * totalsec)) * Math.cos((section2 + 1) * angle)
				p3y = (r2 + (dr * totalsec)) * Math.sin((section2 + 1) * angle)
				p3z = ((section2 + 1) * z0) + height
				p3z2 = ((section2 + 1) * z0) - height

				p4x = (r1 + (dr * totalsec)) * Math.cos((section2 + 1) * angle)
				p4y = (r1 + (dr * totalsec)) * Math.sin((section2 + 1) * angle)
				p4z = (section2 + 1) * z0 + edgeoffset
				p4z2 = (section2 + 1) * z0 - edgeoffset

				pts1 = []
				pts1[0] = [p1x, p1y, p1z] # p1t
				pts1[1] = [p2x, p2y, p2z] # p2t
				pts1[2] = [p3x, p3y, p3z] # p3t
				base[section] = entities.add_face pts1
		
				pts2 = []
				pts2[0] = [p3x, p3y, p3z] # p3t
				pts2[1] = [p4x, p4y, p4z] # p4t
				pts2[2] = [p1x, p1y, p1z] # p1t
				base[section] = entities.add_face pts2
	
				pts3 = []
				pts3[0] = [p1x, p1y, p1z2] # p1b
				pts3[1] = [p2x, p2y, p2z2] # p2b
				pts3[2] = [p3x, p3y, p3z2] # p3b
				base[section] = entities.add_face pts3
		
				pts4 = []
				pts4[0] = [p3x, p3y, p3z2] # p3b
				pts4[1] = [p4x, p4y, p4z2] # p4b
				pts4[2] = [p1x, p1y, p1z2] # p1b
				base[section] = entities.add_face pts4
	
				# p1b p1t p4b
				# p1t p4t p4b
		
				pts5 = []
				pts4[0] = [p1x, p1y, p1z2] # p1b
				pts4[1] = [p1x, p1y, p1z] # p1t
				pts4[2] = [p4x, p4y, p4z2] # p4b
				base[section] = entities.add_face pts4
		
				pts6 = []
				pts6[0] = [p1x, p1y, p1z] # p1t
				pts6[1] = [p4x, p4y, p4z] # p4t
				pts6[2] = [p4x, p4y, p4z2] # p4b
				base[section] = entities.add_face pts6
	
				section += 1
			end

		else

			while section < (totalsec + 1) 
				section2 = section - 1

				p1x = (r1 + (dr * section2)) * Math.cos(section2 * angle)
				p1y = (r1 + (dr * section2)) * Math.sin(section2 * angle)
				p1z = section2 * z0
		
				p2x = (r2 + (dr * section2)) * Math.cos(section2 * angle)
				p2y = (r2 + (dr * section2)) * Math.sin(section2 * angle)
				p2z = (section2 * z0) + height
				p2z2 = (section2 * z0) - height

				p3x = (r2 + (dr * (section2 + 1))) * Math.cos((section2 + 1) * angle)
				p3y = (r2 + (dr * (section2 + 1))) * Math.sin((section2 + 1) * angle)
				p3z = ((section2 + 1) * z0) + height
				p3z2 = ((section2 + 1) * z0) - height

				p4x = (r1 + (dr * (section2 + 1))) * Math.cos((section2 + 1) * angle)
				p4y = (r1 + (dr * (section2 + 1))) * Math.sin((section2 + 1) * angle)
				p4z = (section2 + 1) * z0

				pts1 = []
				pts1[0] = [p1x, p1y, p1z]
				pts1[1] = [p2x, p2y, p2z]
				pts1[2] = [p3x, p3y, p3z]
				base[section] = entities.add_face pts1
		
				pts2 = []
				pts2[0] = [p3x, p3y, p3z]
				pts2[1] = [p4x, p4y, p4z]
				pts2[2] = [p1x, p1y, p1z]
				base[section] = entities.add_face pts2
	
				pts3 = []
				pts3[0] = [p1x, p1y, p1z]
				pts3[1] = [p2x, p2y, p2z2]
				pts3[2] = [p3x, p3y, p3z2]
				base[section] = entities.add_face pts3
		
				pts4 = []
				pts4[0] = [p3x, p3y, p3z2]
				pts4[1] = [p4x, p4y, p4z]
				pts4[2] = [p1x, p1y, p1z]
				base[section] = entities.add_face pts4
	
				section += 1
			end

			while section < (totalsrsec + totalsec + 1) 
				section2 = section - 1
			
				p1x = (r1 + (dr * totalsec)) * Math.cos(section2 * angle)
				p1y = (r1 + (dr * totalsec)) * Math.sin(section2 * angle)
				p1z = section2 * z0
		
				p2x = (r2 + (dr * totalsec)) * Math.cos(section2 * angle)
				p2y = (r2 + (dr * totalsec)) * Math.sin(section2 * angle)
				p2z = (section2 * z0) + height
				p2z2 = (section2 * z0) - height

				p3x = (r2 + (dr * totalsec)) * Math.cos((section2 + 1) * angle)
				p3y = (r2 + (dr * totalsec)) * Math.sin((section2 + 1) * angle)
				p3z = ((section2 + 1) * z0) + height
				p3z2 = ((section2 + 1) * z0) - height

				p4x = (r1 + (dr * totalsec)) * Math.cos((section2 + 1) * angle)
				p4y = (r1 + (dr * totalsec)) * Math.sin((section2 + 1) * angle)
				p4z = (section2 + 1) * z0

				pts1 = []
				pts1[0] = [p1x, p1y, p1z]
				pts1[1] = [p2x, p2y, p2z]
				pts1[2] = [p3x, p3y, p3z]
				base[section] = entities.add_face pts1
		
				pts2 = []
				pts2[0] = [p3x, p3y, p3z]
				pts2[1] = [p4x, p4y, p4z]
				pts2[2] = [p1x, p1y, p1z]
				base[section] = entities.add_face pts2
	
				pts3 = []
				pts3[0] = [p1x, p1y, p1z]
				pts3[1] = [p2x, p2y, p2z2]
				pts3[2] = [p3x, p3y, p3z2]
				base[section] = entities.add_face pts3
		
				pts4 = []
				pts4[0] = [p3x, p3y, p3z2]
				pts4[1] = [p4x, p4y, p4z]
				pts4[2] = [p1x, p1y, p1z]
				base[section] = entities.add_face pts4
	
				section += 1
			end

		end

		model.commit_operation
	end
	def self.drawWhorl_save
	
		chosen_folder = UI.select_directory(title: "Select Image Directory")

		if chosen_folder != nil
			prompts = ["File name"]
			values = ["untitled.WRL"]
			#results = inputbox prompts, values, "Save Settings"
			theFilePath = chosen_folder.to_s + '/'
			results = inputbox prompts, values, theFilePath
			return if not results
			
			theName = results[0]
			
			fullPath = theFilePath.to_s << theName

			prompts = ["Full Path"]
			values = [fullPath]
			results = inputbox prompts, values, "verify"
			
			begin
				file = File.open(fullPath, "w")
				file.puts @@WRLeradius
				file.puts @@WRLsradius
				file.puts @@WRLpitch
				file.puts @@WRLrotations
				file.puts @@WRLsrrotations
				file.puts @@WRLsections
				file.puts @@WRLthreadheight
				file.puts @@WRLthreadlength
				file.puts @@WRLedgeheight
			rescue IOError => e
			  #some error occur, dir not writable etc.
			ensure
			  file.close unless file.nil?
			end
		end
	end
	def self.drawWhorl_help
		dlg = UI::WebDialog.new("Show drawwhorl.com", true, "ShowDrawWhorlDotCom", 739, 641, 150, 150, true);
		dlg.set_url "http://www.drawwhorl.com"
		dlg.show
	end
end
# Create menu items
unless file_loaded?(__FILE__)
 mymenu = UI.menu("Plugins").add_submenu("Draw Whorl, Screw, or Bolt")
 mymenu.add_item("Draw with dialog") {DrawWhorl2::drawWhorl_w_dialog}
 mymenu.add_item("Open/Draw saved settings") {DrawWhorl2::drawWhorl_w_open}
 mymenu.add_item("Save settings") {DrawWhorl2::drawWhorl_save}
 mymenu.add_item("Help/About") {DrawWhorl2::drawWhorl_help}
 file_loaded(__FILE__)
end

